import argparse
import functools
import runpy
import sys
from typing import Any, Callable

lru_cache: Callable[..., Any] = functools.lru_cache
_PACKAGE_NAME: str = __file__.split("/")[-2]


def _get_command() -> str:
    parser: argparse.ArgumentParser = argparse.ArgumentParser(
        description="Parse command-line arguments"
    )
    parser.add_argument(
        "command", help='The name of a command ("clean", "distribute", etc.)?'
    )
    arguments: argparse.Namespace = parser.parse_known_args()[0]
    sys.argv.remove(arguments.command)
    return arguments.command


def main(command: str = "") -> None:
    """
    Run a sub-module corresponding to the indicated `operation`.

    Parameters:

    - command (str): The name of a sub-module to run as "__main__".
    """
    command = command or _get_command()
    module_name: str = f"{_PACKAGE_NAME}.{command}"
    runpy.run_module(module_name, run_name="__main__")


if __name__ == "__main__":
    main()



"""
This module cleans up files which are ignored by git
"""
import argparse
import functools
import os
import shutil
from itertools import chain
from subprocess import getstatusoutput
from typing import Any, Callable, Dict, FrozenSet, Iterable, Sequence, Set

ROOT_DIRECTORY: str = "."
EXCLUDE_DIRECTORIES: FrozenSet[str] = frozenset(
    (
        "./.idea",  # Jetbrains' IDE project settings (Pycharm, Intellij IDEA)
        "./.vscode",  # Microsoft Visual Studio Code project settings
        "./.git",  # Git history
        "./venv",  # Commonly used location for virtual environments
    )
)

lru_cache: Callable[..., Any] = functools.lru_cache


def _run(command: str) -> str:
    status: int
    output: str
    status, output = getstatusoutput(command)
    # Create an error if a non-zero exit status is encountered
    if status:
        raise OSError(output)
    return output


@lru_cache()
def _is_excluded(
    absolute_path: str, exclude_directories: FrozenSet[str] = frozenset()
) -> bool:
    absolute_path = absolute_path.rstrip("/")
    for directory_path in exclude_directories:
        if absolute_path.startswith(f"{directory_path}/"):
            return True
    return False


@lru_cache()
def _absolute_sub_directories(
    root_directory: str, directories: FrozenSet[str]
) -> FrozenSet[str]:
    directory: str
    return frozenset(
        (
            os.path.abspath(os.path.join(root_directory, directory))
            for directory in directories
        )
    )


def get_ignored_files(
    root_directory: str, exclude_directories: FrozenSet[str] = frozenset(),
) -> Iterable[str]:
    """
    Yield the absolute path of all ignored files, excluding those in any of the
    `exclude_directories`.

    Parameters:

    - root_directory (str): The root project directory.
    - exclude_directories ({str}) = frozenset(): A `frozenset` of
      sub-directories to exclude.
    """
    path: str
    directory_name: str
    directories_files: Dict[str, Set[str]] = {}
    paths: Set[str]
    exclude_directory: str
    root_directory = os.path.abspath(root_directory)
    exclude_directories = _absolute_sub_directories(
        root_directory, exclude_directories
    )
    path_prefix: str = f'{root_directory.rstrip("/")}/'
    path_prefix_length: int = len(path_prefix)
    for path in _run(f"git ls-files -o '{root_directory}'").split("\n"):
        path = os.path.abspath(f"./{path}")
        if not _is_excluded(path, exclude_directories):
            directory_name = ""
            if "/" in path[path_prefix_length:]:
                directory_name = path[path_prefix_length:].split("/")[0]
            if directory_name not in directories_files:
                directories_files[directory_name] = set()
            directories_files[directory_name].add(path)
    for directory_name, paths in directories_files.items():
        number_of_files: int = len(paths)
        print(
            f"Found {number_of_files} ignored "
            f'file{"s" if number_of_files > 1 else ""} in ./'
            f"{directory_name}"
        )
        for path in paths:
            yield path


def delete_empty_directories(
    root_directory: str,
    exclude_directories: FrozenSet[str] = frozenset(),
    _recurrence: bool = True,
) -> int:
    """
    Deletes empty directories under the current directory.

    Parameters:

    - exclude_directories ({str}) = frozenset():
      A set of top-level directory names to exclude
    """
    number_of_deleted_directories: int = 0
    root: str
    directories: Sequence[str]
    files: Sequence[str]
    if not _recurrence:
        root_directory = os.path.abspath(root_directory)
        exclude_directories = _absolute_sub_directories(
            root_directory, exclude_directories
        )
    for root, directories, files in os.walk(root_directory, topdown=False):
        root = os.path.abspath(root)
        if not _is_excluded(root, exclude_directories):
            if not any(
                filter(
                    lambda name: name != ".DS_Store", chain(directories, files)
                )
            ):
                shutil.rmtree(root)
                number_of_deleted_directories += 1
    if number_of_deleted_directories:
        number_of_deleted_directories += delete_empty_directories(
            root_directory,
            exclude_directories=exclude_directories,
            _recurrence=False,
        )
    if _recurrence and number_of_deleted_directories:
        print(f"Deleted {number_of_deleted_directories} empty directories")
    return number_of_deleted_directories


def delete_ignored(
    root_directory: str, exclude_directories: FrozenSet[str] = frozenset(),
) -> None:
    """
    Delete files which are ignored by Git.

    Parameters:

    - root_directory (str): The root project directory.
    - exclude_directories ({str}) = frozenset(): A `frozenset` of directories
      which should not be touched.
    """
    for path in get_ignored_files(
        root_directory=root_directory, exclude_directories=exclude_directories
    ):
        os.remove(path)


def main(
    root_directory: str = ".",
    exclude_directories: FrozenSet[str] = EXCLUDE_DIRECTORIES,
) -> None:
    """
    Cleanup (delete) files which are ignored by Git and subsequently delete all
    empty directories.

    Parameters:

    - root_directory (str) = ".": The project's root directory.
    - exclude_directories ({{str}}) = {EXCLUDE_DIRECTORIES}: A `frozenset` of
      directories to leave untouched.
    """
    delete_ignored(root_directory, exclude_directories=exclude_directories)
    delete_empty_directories(
        root_directory, exclude_directories=exclude_directories
    )


main.__doc__ = main.__doc__.format(  # type: ignore
    EXCLUDE_DIRECTORIES=EXCLUDE_DIRECTORIES
)


if __name__ == "__main__":
    parser: argparse.ArgumentParser = argparse.ArgumentParser(
        description='Parse command-line arguments for a "clean" operation'
    )
    parser.add_argument(
        "--exclude",
        "-e",
        action="store",
        type=str,
        default=None,
        help=(
            'A path list of sub-directories to exclude (separated by ":" or '
            '";", depending on the operating system).'
        ),
    )
    parser.add_argument(
        "root",
        type=str,
        default=".",
        nargs="?",
        help="The root directory path for the project.",
    )
    arguments: argparse.Namespace = parser.parse_args()
    main(
        root_directory=arguments.root,
        exclude_directories=(
            frozenset(os.path.split(arguments.exclude))
            if arguments.exclude
            else EXCLUDE_DIRECTORIES
        ),
    )



import functools
import importlib
import os
import runpy
import sys
from distutils.core import run_setup
from itertools import chain
from time import time
from types import ModuleType
from typing import (
    Any,
    Callable,
    Dict,
    FrozenSet,
    Iterable,
    List,
    Optional,
    Tuple,
    Union,
)

lru_cache: Callable[..., Any] = functools.lru_cache


def _list_dist(root: str, modified_at_or_after: float = 0.0) -> FrozenSet[str]:
    dist_root: str = os.path.join(root, "dist")
    dist_file: str
    dist_sub_directories: List[str]
    dist_files: Iterable[str]
    try:
        dist_root, dist_sub_directories, dist_files = next(
            iter(os.walk(dist_root))
        )
    except StopIteration:
        raise FileNotFoundError(
            f"No distributions could be found in {dist_root}"
        )
    dist_files = (
        os.path.join(dist_root, dist_file) for dist_file in dist_files
    )
    if modified_at_or_after:
        dist_files = filter(
            lambda dist_file: (  # noqa
                os.path.getmtime(dist_file) >= modified_at_or_after
            ),
            dist_files,
        )
    try:
        return frozenset(dist_files)
    except (NotADirectoryError, FileNotFoundError):
        return frozenset()


def _setup(root: str) -> FrozenSet[str]:
    start_time: float = time()
    current_directory: str = os.path.curdir
    os.chdir(root)
    abs_setup: str = os.path.join(root, "setup.py")
    setup_args: List[str] = ["sdist", "bdist_wheel"]
    print(f'{sys.executable} {abs_setup} {" ".join(setup_args)}')
    try:
        run_setup(abs_setup, setup_args)
    finally:
        os.chdir(current_directory)
    return _list_dist(root, modified_at_or_after=start_time)


@lru_cache()
def _get_credentials_from_cerberus() -> Tuple[Optional[str], Optional[str]]:
    """
    If `--cerberus-url` and `--cerberus-path` keyword arguments are provided,
    retrieve the repository credentials and store them in the "TWINE_USERNAME"
    and "TWINE_PASSWORD" environment variables.
    """
    username: Optional[str] = _argv_pop(
        sys.argv, "-u", _argv_pop(sys.argv, "--username")
    )
    password: Optional[str] = None
    cerberus_url: Optional[str] = _argv_pop(sys.argv, "--cerberus-url")
    cerberus_path: Optional[str] = _argv_pop(sys.argv, "--cerberus-path")
    if cerberus_url and cerberus_path:
        # Only import the Cerberus utility if/when the "--cerberus-url"
        # and "--cerberus-path" keyword arguments are provided, as the
        # "cerberus-python-client" and "boto3" libraries needed for this
        # are optional (only installed with the package extra
        # "daves-dev-tools[cerberus]")
        cerberus: ModuleType = importlib.import_module(
            ".utilities.cerberus",
            (
                os.path.split(os.path.dirname(__file__))[-1]
                if __name__ == "__main__"
                else ".".join(__name__.split(".")[:-1])
            ),
        )
        if not username:
            # If no username is provided, we assume the username has
            # been concatenated with the path, and extract it
            cerberus_path_list: List[str] = cerberus_path.split("/")
            username = cerberus_path_list.pop()
            cerberus_path = "/".join(cerberus_path_list)
        secrets: Union[Dict[str, str], str, None] = getattr(
            cerberus, "get_cerberus_secrets"
        )(cerberus_url, cerberus_path)
        if secrets is not None:
            assert isinstance(secrets, dict)
            password = secrets[username]
            sys.argv += ["--username", username]
            sys.argv += ["--password", password]
    return username, password


def _dist(root: str, distributions: FrozenSet[str]) -> None:
    argv: List[str] = sys.argv
    twine_argv: List[str] = list(
        chain(sys.argv[:1], ["upload"], sys.argv[1:], sorted(distributions))
    )
    current_directory: str = os.path.curdir
    os.chdir(root)
    print(" ".join(["twine"] + twine_argv[1:]))  # type: ignore
    try:
        sys.argv = twine_argv
        runpy.run_module("twine", run_name="__main__")
    finally:
        os.chdir(current_directory)
        sys.argv = argv


def _cleanup(root: str) -> None:
    current_directory: str = os.path.curdir
    os.chdir(root)
    try:
        run_setup(os.path.join(root, "setup.py"), ["clean", "--all"])
    finally:
        os.chdir(current_directory)


def _argv_positional_last_index(argv: List[str]) -> Optional[int]:
    index: int
    for index in range(len(argv) - 1, 0, -1):
        if not (
            argv[index].startswith("-") or argv[index - 1].startswith("-")
        ):
            return index
    return None


def _argv_pop_last_positional(argv: List[str], default: str = "") -> str:
    index: Optional[int] = _argv_positional_last_index(argv)
    if index is not None:
        return argv.pop(index)
    return default


def _argv_pop(
    argv: List[str], key: str, default: Optional[str] = None
) -> Optional[str]:
    key_index: int
    value: Optional[str] = default
    # Ensure we are looking for a keyword argument
    assert key.startswith("-")
    try:
        key_index = argv.index(key)
        # Ensure there is a value
        assert len(argv) > key_index + 1
        value = argv.pop(key_index + 1)
        argv.pop(key_index)
    except ValueError:
        pass
    return value


def main(root: str = "") -> None:
    _get_credentials_from_cerberus()
    root = root or _argv_pop_last_positional(sys.argv, ".")
    root = os.path.abspath(root).rstrip("/")
    try:
        _dist(root, _setup(root))
    finally:
        _cleanup(root)


if __name__ == "__main__":
    main()



import re
import sys
from collections import OrderedDict
from itertools import chain
from typing import (
    Dict,
    Iterable,
    List,
    Match,
    Optional,
    Pattern,
    Sequence,
    Set,
)

import setuptools

_INSTALL_REQUIRES: str = "install_requires"


_extras_pattern: Pattern = re.compile(r"^([^\[]+\[)([^\]]+)(\].*)$")


def consolidate_requirement_options(
    requirements: Iterable[str],
) -> Iterable[str]:
    requirement: str
    templates_options: Dict[str, Set[str]] = OrderedDict()
    traversed_requirements: Set[str] = set()
    for requirement in requirements:
        print(requirement)
        match: Optional[Match] = _extras_pattern.match(requirement)
        if match:
            groups: Sequence[str] = match.groups()
            no_extras_requirement: str = f"{groups[0][:-1]}{groups[2][1:]}"
            template: str = f"{groups[0]}{{}}{groups[2]}"
            if template not in templates_options:
                templates_options[template] = set()
            templates_options[template] |= set(groups[1].split(","))
            if no_extras_requirement in templates_options:
                del templates_options[no_extras_requirement]
        elif requirement not in traversed_requirements:
            templates_options[requirement] = None
    template: str
    options: Optional[List[str]]
    for template, options in templates_options.items():
        if options:
            yield template.format(",".join(sorted(options)))
        else:
            yield template


def setup(**kwargs) -> None:
    """
    This `setup` script intercepts arguments to be passed to
    `setuptools.setup` in order to dynamically alter setup requirements
    while retaining a function call which can be easily parsed and altered
    by `setuptools-setup-versions`.
    """
    # Require the package "dataclasses" for python 3.6, but not later
    # python versions (since it's part of the standard library after 3.6)
    if sys.version_info[:2] == (3, 6):
        if _INSTALL_REQUIRES not in kwargs:
            kwargs[_INSTALL_REQUIRES] = []
        kwargs[_INSTALL_REQUIRES].append("dataclasses")
    # Add an "all" extra which includes all extra requirements
    if "extras_require" in kwargs:
        if "all" not in kwargs["extras_require"]:
            kwargs["extras_require"]["all"] = list(
                consolidate_requirement_options(
                    chain(*kwargs["extras_require"].values())
                )
            )
        kwargs["extras_require"]["test"] = list(
            consolidate_requirement_options(
                chain(
                    *(
                        values
                        for key, values in kwargs["extras_require"].items()
                        if key not in ("dev", "all")
                    )
                )
            )
        )
        print(
            "extras_require[all]:\n"
            + "\n".join(
                f"- {requirement}"
                for requirement in kwargs["extras_require"]["all"]
            )
        )
        print(
            "extras_require[test]:\n"
            + "\n".join(
                f"- {requirement}"
                for requirement in kwargs["extras_require"]["test"]
            )
        )
    # Pass the modified keyword arguments to `setuptools.setup`
    setuptools.setup(**kwargs)


setup(
    name="daves-dev-tools",
    version="0.3.0",
    description="Dave's Dev Tools",
    author="David Belais",
    author_email="david@belais.me",
    python_requires="~=3.6",
    packages=["daves_dev_tools", "daves_dev_tools.utilities"],
    package_data={
        "daves_dev_tools": ["py.typed"],
        "daves_dev_tools.utilities": ["py.typed"],
    },
    install_requires=["twine~=3.2", "wheel~=0.36"],
    extras_require={
        "cerberus": ["cerberus-python-client~=2.5"],
        "dev": [
            "black~=19.10b0",
            "readme-md-docstrings>=0.1.0,<1",
            "setuptools-setup-versions>=1.4.1,<2",
        ],
        "test": ["pytest~=5.4", "tox~=3.20", "flake8~=3.8", "mypy~=0.790"],
    },
    entry_points={
        "console_scripts": ["daves-dev-tools = daves_dev_tools.__main__:main"]
    },
)



import logging
import sys
from warnings import warn
from collections import OrderedDict
from traceback import format_exception
from typing import Dict, List, Optional

import boto3  # type: ignore
from botocore.exceptions import (  # type: ignore
    ClientError,
    NoCredentialsError,
)
from cerberus import CerberusClientException  # type: ignore
from cerberus.client import CerberusClient  # type: ignore

__all__: List[str] = [
    "get_cerberus_secrets",
]


def get_cerberus_secrets(cerberus_url: str, path: str,) -> Dict[str, str]:
    """
    This function attempts to access Cerberus secrets at the given `path`
    with each AWS profile until successful, or until having run out of
    profiles, and returns the secrets if successful (or raises an error if
    not).

    Parameters:

    - **cerberus_url** (str): The Cerberus API endpoint
    - **path** (str): The Cerberus path containing the desired secret(s)
    """
    if (boto3 is None) or (CerberusClient is None):
        raise RuntimeError(
            "The `boto3` and `cerberus-python-client` packages must be "
            "installed in order to use "
            "`nike.analytics_orm.utilities.get_cerberus_secrets`. "
            'Install the package option "nike-analytics-orm[cerberus]" '
            "in order to use this function."
        )
    secrets: Optional[Dict[str, str]] = None
    errors: Dict[str, str] = OrderedDict()
    for profile_name in boto3.Session().available_profiles + [None]:
        arn: str = ""
        try:
            session: boto3.Session = boto3.Session(profile_name=profile_name)
            arn = session.client("sts").get_caller_identity().get("Arn")
            secrets = CerberusClient(
                cerberus_url, aws_session=session
            ).get_secrets_data(path)
            break
        except (CerberusClientException, ClientError, NoCredentialsError):
            errors[arn or profile_name] = "".join(
                format_exception(*sys.exc_info())
            )
    if secrets is None:
        error_text: str = "\n".join(
            f'{key or "[default]"}:\n{value}\n'
            for key, value in errors.items()
        )
        warn(error_text)
        logging.warning(error_text)
        raise PermissionError(
            "No AWS profile was found with access to the requested secrets"
        )
    return secrets
